@@ -13,9 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Authors: Christian Kellner <gicmo@gnome.org>
  *          Samuel Cormier-Iijima <sciyoshi@gmail.com>
@@ -395,6 +393,8 @@ g_inet_address_new_from_string (const gchar *string)
   struct in6_addr in6_addr;
 #endif
 
+  g_return_val_if_fail (string != NULL, NULL);
+
   /* If this GInetAddress is the first networking-related object to be
    * created, then we won't have called g_networking_init() yet at
    * this point.
